package server

import (
	"fmt"
	"game/common/proto/pb"
	"game/common/serviceName"
	"game/common/userBindService"
	"game/server/chat/config"
	"game/server/chat/model"
	"github.com/fox/fox/ipb"
	"github.com/fox/fox/log"
	"github.com/fox/fox/processor"
	"github.com/fox/fox/service"
	"github.com/golang/protobuf/proto"
)

var Chat []*ChatService

type ChatService struct {
	*service.NatsService
	processor   *processor.Processor
	bindService *userBindService.UserBindService
}

func Init() {
	for i := 0; i < config.Command.ServiceNum; i++ {
		sid := config.Command.ServiceId + i
		if srv := newChatService(sid); srv != nil {
			Chat = append(Chat, srv)
		}
	}
}

func Stop() {
	for _, srv := range Chat {
		srv.NotifyStop()
	}
	for _, srv := range Chat {
		srv.WaitStop()
	}
}

func newChatService(serviceId int) *ChatService {
	var err error
	s := new(ChatService)

	sName := fmt.Sprintf("%v-%d", serviceName.Chat, serviceId)
	if s.NatsService, err = service.NewNatsService(&service.InitNatsServiceParams{
		EtcdAddress:  config.Cfg.Etcd.Address,
		EtcdUsername: "",
		EtcdPassword: "",
		NatsAddress:  config.Cfg.Nats.Address,
		ServiceType:  serviceName.Chat,
		ServiceName:  sName,
		OnFunc:       s,
		TypeId:       int(pb.ServiceTypeId_STI_Chat),
		Version:      config.Cfg.BuildDate,
	}); err != nil {
		log.Fatal(err.Error())
		return nil
	}

	s.bindService = userBindService.NewUserBindService(model.UserRedis, s.ServiceEtcd())
	s.processor = processor.NewProcessor()
	s.initProcessor()
	s.OnInit()
	return s
}

func (s *ChatService) OnInit() {
	// if err := s.NatsService.QueueSubscribe(service.GroupTopic(s), service.GroupQueue(s)); err != nil {
	// 	log.Error(err.Error())
	// }
	s.NatsService.Run()
	log.Debug("onInit")
}

func (s *ChatService) CanStop() bool {
	//log.Debug("chatService.CanStop")
	return true
}

func (s *ChatService) OnStop() {
	s.NatsService.OnStop()
	log.Debug("OnStop")
}

// 处理其它服发送过来的消息
func (s *ChatService) OnMessage(data []byte) error {
	var iMsg = &ipb.InternalMsg{}
	var err error
	if err = proto.Unmarshal(data, iMsg); err != nil {
		log.Error(err.Error())
		return err
	}
	if req, err := s.processor.Unmarshal(iMsg.MsgId, iMsg.Msg); err == nil {
		err = s.processor.Dispatch(iMsg.MsgId, iMsg.UserId, req)
	} else {
		log.Error(err.Error())
	}
	//log.Debug(s.Log("received message:%v", iMsg.MsgId))
	return nil
}

// 向内部服务发送消息
func (s *ChatService) SendServiceData(topic string, userId int64, msgId int32, data []byte) {
	iMsg := ipb.MakeMsg(s.Name(), 0, userId, msgId, data)
	_ = s.Send(topic, iMsg)
}

// 向内部服务发送消息
func (s *ChatService) SendServiceMsg(topic string, userId int64, msgId int32, msg proto.Message) {
	data, _ := proto.Marshal(msg)
	s.SendServiceData(topic, userId, msgId, data)
}
