package server

import (
	"game/common/proto/pb"
	"game/common/rpc"
	"game/server/colorgame/room"
	"github.com/fox/fox/ipb"
	"github.com/fox/fox/ksync"
	"github.com/fox/fox/processor"
	"github.com/fox/fox/service"
)

const (
// timeout = time.Second * 30
)

func (s *ColorService) initProcessor() {
	s.processor.RegisterMessages(processor.RegisterMetas{
		pb.MsgId_ReqMatchRoomId: {pb.ReqMatchRoom{}, s.onEnterRoom},
	})
}

// 登录或注册
func (s *ColorService) onEnterRoom(iMsg *ipb.InternalMsg, req *pb.ReqMatchRoom) {
	ksync.GoSafe(func() {
		us, code := rpc.RpcGetGameUser(s.bindService, s, iMsg.UserId)
		if code != pb.ErrCode_OK {
			s.SendServiceMsg(service.TopicEx(iMsg.ServiceName), iMsg.ConnId, iMsg.UserId,
				int32(pb.MsgId_RspMatchRoomId), &pb.RspMatchRoom{Code: pb.ErrCode_SystemErr})
		}
		// 切回服务协程处理业务逻辑
		s.RunOnce(func() {
			colorUser := room.NewColorPlayer(service.TopicEx(iMsg.ServiceName), s.room.Id(), &us.User, &us.UserResources)
			s.playerMgr.Add(colorUser)
			_ = s.room.Dispatch(colorUser, iMsg.MsgId, iMsg, req)
		})

	}, nil)
}
