package server

import (
	"fmt"
	"game/common/proto/pb"
	"game/common/serviceName"
	"game/common/userBindService"
	"game/server/login/config"
	"game/server/login/model"
	"github.com/fox/fox/ipb"
	"github.com/fox/fox/log"
	"github.com/fox/fox/processor"
	"github.com/fox/fox/service"
	"github.com/golang/protobuf/proto"
)

var Login []*LoginService

type LoginService struct {
	*service.NatsService
	processor   *processor.Processor
	bindService *userBindService.UserBindService
}

func Init() {
	log.DebugF("init service begin id:%v, num:%v", config.Command.ServiceId, config.Command.ServiceNum)
	for i := 0; i < config.Command.ServiceNum; i++ {
		sid := config.Command.ServiceId + i
		if srv := newLoginService(sid); srv != nil {
			Login = append(Login, srv)
		}
	}
}

func Stop() {
	for _, srv := range Login {
		log.DebugF("notify stop service %v", srv.Name())
		srv.NotifyStop()
	}
	for _, srv := range Login {
		srv.WaitStop()
	}
}

func newLoginService(serviceId int) *LoginService {
	var err error
	s := new(LoginService)

	sName := fmt.Sprintf("%v-%d", serviceName.Login, serviceId)
	if s.NatsService, err = service.NewNatsService(&service.InitNatsServiceParams{
		EtcdAddress:  config.Cfg.Etcd.Address,
		EtcdUsername: "",
		EtcdPassword: "",
		NatsAddress:  config.Cfg.Nats.Address,
		ServiceType:  serviceName.Login,
		ServiceName:  sName,
		OnFunc:       s,
		TypeId:       int(pb.ServiceTypeId_STI_Login),
		Version:      config.Cfg.BuildDate,
	}); err != nil {
		log.Fatal(err.Error())
		return nil
	}

	s.bindService = userBindService.NewUserBindService(model.UserBindServiceRedis, s.ServiceEtcd())
	s.processor = processor.NewProcessor()
	s.initProcessor()
	s.OnInit()
	return s
}

func (s *LoginService) OnInit() {
	// if err := s.NatsService.QueueSubscribe(service.GroupTopic(s), service.GroupQueue(s)); err != nil {
	// 	log.Error(err.Error())
	// }
	s.NatsService.Run()
	log.Debug("onInit")
}

func (s *LoginService) CanStop() bool {
	return true
}

func (s *LoginService) OnStop() {
	s.NatsService.OnStop()
	log.Debug("OnStop")
}

// 处理其它服发送过来的消息
func (s *LoginService) OnMessage(data []byte) error {
	var iMsg = &ipb.InternalMsg{}
	var err error
	if err = proto.Unmarshal(data, iMsg); err != nil {
		log.Error(err.Error())
		return err
	}
	if req, err := s.processor.Unmarshal(iMsg.MsgId, iMsg.Msg); err == nil {
		err = s.processor.Dispatch(iMsg.MsgId, iMsg, req)
	} else {
		log.Error(err.Error())
	}
	log.Debug(s.Log("received message:%v", iMsg.MsgId))
	return nil
}

// 向内部服务发送消息
func (s *LoginService) SendServiceData(topic string, connId uint32, userId int64, msgId int32, data []byte) {
	iMsg := ipb.MakeMsg(s.Name(), connId, userId, msgId, data)
	_ = s.Send(topic, iMsg)
}

// 向内部服务发送消息
func (s *LoginService) SendServiceMsg(topic string, connId uint32, userId int64, msgId int32, msg proto.Message) {
	data, _ := proto.Marshal(msg)
	s.SendServiceData(topic, connId, userId, msgId, data)
}
