package server

import (
	"game/common/model/user"
	"game/common/proto/pb"
	"game/common/rpc"
	"game/common/utils"
	"github.com/fox/fox/ipb"
	"github.com/fox/fox/ksync"
	"github.com/fox/fox/log"
	"github.com/fox/fox/service"
)

// 匹配房间
func (s *MatchService) onMatchRoom(iMsg *ipb.InternalMsg, req *pb.ReqMatchRoom) {
	ksync.GoSafe(func() {
		// color game无需进入匹配队列
		playType := pb.ServiceTypeId(req.PlayType)
		switch playType {
		case pb.ServiceTypeId_STI_ColorGame:
			node, err := s.bindService.RandServiceNode(playType)
			if err != nil {
				log.ErrorF("db service node error:%v", err)
				s.SendServiceMsg(service.TopicEx(iMsg.ServiceName), iMsg.ConnId, iMsg.UserId,
					int32(pb.MsgId_RspMatchRoomId), &pb.RspMatchRoom{Code: pb.ErrCode_SystemErr})
				return
			}
			s.SendServiceMsg(service.TopicEx(node.Name), iMsg.ConnId, iMsg.UserId, int32(pb.MsgId_ReqMatchRoomId), req)
			return
		}

		var us *user.GameUser
		rsp := &pb.RspMatchRoom{}
		us, rsp.Code = rpc.RpcGetGameUser(s.bindService, s, iMsg.UserId)
		if rsp.Code != pb.ErrCode_OK {
			s.SendServiceMsg(service.TopicEx(iMsg.ServiceName), iMsg.ConnId, iMsg.UserId, int32(pb.MsgId_RspMatchRoomId), rsp)
			return
		}
		vipLv, vipExp := utils.VipLevel(us.VipExp)
		rsp.User = &pb.GameUser{
			UserId:      us.ID,
			Nickname:    us.Nickname,
			Avatar:      us.AvatarUrl,
			AvatarFrame: us.AvatarFrame,
			VipLevel:    vipLv,
			VipExp:      vipExp,
			Seat:        0,
			Gold:        us.Gold,
		}
		rsp.PlayType = req.PlayType
		rsp.RoomType = req.RoomType
		switch pb.ServiceTypeId(rsp.PlayType) {
		case pb.ServiceTypeId_STI_ColorGame:
			rsp.ColorInfo = &pb.RspMatchRoom_ColorInfo{}
		}

		s.SendServiceMsg(service.TopicEx(iMsg.ServiceName), iMsg.ConnId, iMsg.UserId, int32(pb.MsgId_RspMatchRoomId), rsp)
	}, nil)
}
